/*
 * Copyright Bigeon Emmanuel (2014)
 *
 * emmanuel@bigeon.fr
 *
 * This software is a computer program whose purpose is to
 * provide a generic framework for console applications.
 *
 * This software is governed by the CeCILL license under French law and
 * abiding by the rules of distribution of free software.  You can  use,
 * modify and/or redistribute the software under the terms of the CeCILL
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * "http://www.cecill.info".
 *
 * As a counterpart to the access to the source code and  rights to copy,
 * modify and redistribute granted by the license, users are provided only
 * with a limited warranty  and the software's author,  the holder of the
 * economic rights,  and the successive licensors  have only  limited
 * liability.
 *
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer knowledge. Users are therefore
 * encouraged to load and test the software's suitability as regards their
 * requirements in conditions enabling the security of their systems and/or
 * data to be ensured and,  more generally, to use and operate it in the
 * same conditions as regards security.
 *
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL license and that you accept its terms.
 */
/**
 * acide:fr.bigeon.acide.Command.java
 * Created on: Jul 31, 2014
 */
package fr.bigeon.gclc.command;

import java.io.IOException;

import fr.bigeon.gclc.manager.ConsoleManager;

/** <p>
 * A command to execute. It is mandatory that it has a name and that name cannot
 * start with minus character or contain spaces.
 * <p>
 * A command can be executed, with parameters that will be provided as an array
 * of strings.
 * <p>
 * The help mechanism is doing the following:
 * <ul>
 * <li>Print the command name
 * <li>Print the {@link #brief()} message
 * <li>Print a blank line
 * <li>Print "Usage:"
 * <li>Print the {@link #usagePattern() usage pattern}
 * <li>Print the {@link #usageDetail() usage details}
 * </ul>
 * <p>
 * The default behavior for the brief message is to print the tip preceeded by a
 * couple of spaces.
 *
 * @author Emmanuel BIGEON */
public abstract class Command implements ICommand {

    /**
     *
     */
    private static final String EOL_LINUX = "\n"; //$NON-NLS-1$
    /** The name of the command */
    protected final String name;

    /** @param name the command name */
    public Command(String name) {
        super();
        this.name = name;
    }

    /** @return a brief description of the command */
    protected String brief() {
        return "  " + tip(); //$NON-NLS-1$
    }

    /* (non-Javadoc)
     * @see fr.bigeon.gclc.command.ICommand#getCommandName() */
    @Override
    public final String getCommandName() {
        return name;
    }

    /* (non-Javadoc)
     * @see fr.bigeon.gclc.command.ICommand#help(fr.bigeon.gclc.ConsoleManager,
     * java.lang.String) */
    @Override
    public final void help(ConsoleManager manager,
                           String... args) throws IOException {
        manager.println(getCommandName());
        manager.println(brief());
        manager.println();
        manager.println("Usage:"); //$NON-NLS-1$
        manager.println(usagePattern());
        manager.println();
        final String details = usageDetail();
        if (details != null && !details.isEmpty()) {
            manager.print(details);
            if (!(details.endsWith(EOL_LINUX) ||
                  details.endsWith(System.lineSeparator()))) {
                manager.println();
            }
        }
    }

    /** <p>
     * This method return the detail of the help. It immediatly follows the
     * {@link #usagePattern() usage pattern}.
     *
     * @return the detailed help (should end with end of line or be empty) */
    @SuppressWarnings("static-method")
    protected String usageDetail() {
        return new String();
    }

    /** <p>
     * This prints the usage pattern for the command. It follows the brief
     * introduction on the command ({@link #brief()})
     *
     * @return the usage pattern */
    protected String usagePattern() {
        return getCommandName();
    }
}
